from sqlalchemy import delete, insert, select, update
from sqlalchemy.ext.asyncio.session import AsyncSession

from ..models import UserModel
from ..schemas.user_schema import UserDBDTO, UserReadDTO


class UserRepository:
    def __init__(self, session: AsyncSession):
        self.session = session

    async def add_one(
        self,
        data: UserDBDTO,
    ) -> UserReadDTO:
        stmt = insert(UserModel).values(**data.model_dump()).returning(UserModel)
        res = await self.session.execute(stmt)
        return UserReadDTO.model_validate(res.scalar_one())

    async def find_all(self) -> list[UserReadDTO]:
        stmt = select(UserModel)
        res = await self.session.execute(stmt)
        res = [UserReadDTO.model_validate(row) for row in res.scalars().all()]
        return res

    async def find_one(self, filter: dict) -> UserReadDTO | None:
        stmt = select(UserModel).filter_by(**filter)
        res = await self.session.execute(stmt)
        res = res.scalar_one_or_none()
        print(res)

        if res is not None:
            return UserReadDTO.model_validate(res)
        return None

    async def patch_one(self, filter: dict, data: UserDBDTO) -> UserReadDTO | None:
        stmt = update(UserModel).where(UserModel.id == filter["id"]).values(**data.model_dump()).returning(UserModel)
        res = await self.session.execute(stmt)
        res = res.scalar_one_or_none()

        if res is not None:
            return UserReadDTO.model_validate(res)
        return None

    async def delete_one(self, filter: dict) -> None:
        stmt = delete(UserModel).filter_by(**filter)
        await self.session.execute(stmt)
        return None
